export declare type ChartFromPointsProps = {
    type: string;
    options?: GoogleChartOptions;
    points?: Array<{
        x: any;
        y: any;
        label: any;
        type: string;
    }>;
    containerId?: string;
    onReady?: () => any;
    onError?: () => any;
    onSelect?: (selection: any) => any;
};
export interface ChartWrapperProps {
    chartType: GoogleChartWrapperChartType;
    containerId?: string;
    options?: GoogleChartOptions;
    dataTable?: {};
    dataSourceUrl?: string;
    query?: string;
    refreshInterval?: number;
    view?: any[] | {};
    render?: (props: ChartWrapperProps, chartWrapper: GoogleChartWrapper) => any;
    children?: (props: ChartWrapperProps, chartWrapper: GoogleChartWrapper) => any;
    onReady?: (chartWrapper: GoogleChartWrapper, dataTable: GoogleDataTable, arrayToDataTable: GoogleArrayToDataTable) => any;
}
export declare type VizEventsProps = {
    chartWrapper: GoogleChartWrapper;
    onReady?: (chartWrapper: GoogleChartWrapper) => any;
    onError?: (chartWrapper: GoogleChartWrapper) => any;
    onSelect?: (selection: Array<{
        row?: any;
        column?: any;
    }>) => any;
    render?: (props: VizEventsProps, chartWrapper: GoogleChartWrapper) => any;
    children?: (props: VizEventsProps, chartWrapper: GoogleChartWrapper) => any;
};
export declare type ChartFromDataTableProps = {
    dataTable: any[][];
    options: GoogleChartOptions;
    containerId?: string | undefined;
    type: string;
};
export declare type GoogleChartLoaderProps = {
    window?: any;
    version?: string | GoogleChartVersion;
    language?: string;
    packages?: GoogleChartPackages[];
    mapsApiKey?: string;
    render: (propsAndState: GoogleChartLoaderPropsAndState) => any;
    renderChart?: (propsAndState: GoogleChartLoaderPropsAndState) => any;
    renderLoader?: (propsAndState: GoogleChartLoaderPropsAndState) => any;
    renderError?: (propsAndState: GoogleChartLoaderPropsAndState) => any;
};
export declare type GoogleChartLoaderPropsAndState = {
    props: GoogleChartLoaderProps;
    state: GoogleChartLoaderState;
};
export declare type GoogleChartLoaderState = {
    isLoaded: boolean;
    isLoading: boolean;
    hasErrored: boolean;
    errorMessage: string;
};
export declare enum GoogleChartWrapperChartType {
    AnnotationChart = "AnnotationChart",
    AreaChart = "AreaChart",
    BarChart = "BarChart",
    BubbleChart = "BubbleChart",
    Calendar = "Calendar",
    CandlestickChart = "CandlestickChart",
    ColumnChart = "ColumnChart",
    ComboChart = "ComboChart",
    DiffChart = "DiffChart",
    DonutChart = "DonutChart",
    Gantt = "Gantt",
    Gauge = "Gauge",
    GeoChart = "GeoChart",
    Histogram = "Histogram",
    LineChart = "LineChart",
    Map = "Map",
    OrgChart = "OrgChart",
    PieChart = "PieChart",
    Sankey = "Sankey",
    ScatterChart = "ScatterChart",
    SteppedAreaChart = "SteppedAreaChart",
    Table = "Table",
    TreeMap = "TreeMap",
    WaterfallChart = "WaterfallChart",
    WordTree = "WordTree",
}
export interface ChartWrapperOptions {
    chartType: string;
    containerId: string;
    options?: {
        width: number;
        height: number;
        is3D: boolean;
        title: string;
    };
    dataTable?: {};
    dataSourceUrl?: string;
    query?: string;
    refreshInterval?: number;
    view: any[] | {};
}
export declare type GoogleChartWrapper = {
    draw: (chartArgs?: ChartWrapperProps) => any;
    toJSON: () => string;
    clone: () => GoogleChartWrapper;
    getDataSourceUrl: () => string;
    getDataTable: () => GoogleDataTable;
    getChartType: () => GoogleChartWrapperChartType;
    getChartName: () => string;
    getChart: () => {
        getSelection: () => {
            row?: any;
            column?: any;
        }[];
    };
    getContainerId: () => string;
    getQuery: () => string;
    getRefreshInterval: () => number;
    getOption: (key: string, opt_default_value?: any) => any;
    getOptions: () => {};
    getSelection: () => {
        row?: any;
        column?: any;
    }[];
    getView: () => {} | any[];
    setDataSourceUrl: (url: string) => void;
    setDataTable: (table: any) => void;
    setChartType: (chartType: GoogleChartWrapperChartType) => void;
    setChartName: (name: string) => void;
    setContainerId: (id: string) => void;
    setQuery: (query_string: string) => void;
    setRefreshInterval: (interval: number) => void;
    setOption: (key: string, value: any) => void;
    setOptions: (options_obj: ChartWrapperProps["options"]) => void;
};
export declare enum GoogleVizEventName {
    ready = "ready",
    error = "error",
    select = "select",
}
export declare type GoogleVizEvents = {
    addListener: (chartWrapper: GoogleChartWrapper, name: GoogleVizEventName, onEvent: (eventArgs: {}) => any) => any;
    removeAllListeners: (chartWrapper: GoogleChartWrapper) => any;
};
export declare enum GoogleChartPackages {
    corechart = "corechart",
    calendar = "calendar",
    gantt = "gantt",
    gauge = "gauge",
    geochart = "geochart",
    map = "map",
    orgchart = "orgchart",
    sankey = "sankey",
    table = "table",
    timeline = "timeline",
    treemap = "treemap",
    wordtree = "wordtree",
}
export declare enum GoogleChartVersion {
    current = "current",
    upcoming = "upcoming",
}
export declare enum GoogleDataTableColumnType {
    "string" = "string",
    "number" = "number",
    "boolean" = "boolean",
    "date" = "date",
    "datetime" = "datetime",
    "timeofday" = "timeofday",
}
export declare enum GoogleDataTableColumnRoleType {
    annotation = "annotation",
    annotationText = "annotationText",
    certainty = "certainty",
    emphasis = "emphasis",
    interval = "interval",
    scope = "scope",
    style = "style",
    tooltip = "tooltip",
    domain = "domain",
}
export declare type GoogleDataTableColumn = {
    type: GoogleDataTableColumnType;
    label?: string;
    role?: GoogleDataTableColumnRoleType;
    pattern?: string;
    p?: {};
    id?: string;
};
export declare type GoogleDataTableCell = {
    v?: any;
    f?: string;
    p?: {};
} | string | number | boolean;
export declare type GoogleDataTableRow = GoogleDataTableCell[];
export declare type GoogleDataTableJS = {
    cols: GoogleDataTableColumn[];
    rows: {
        c: GoogleDataTableRow;
    }[];
    p?: {};
};
export declare type GoogleDataTableRowFilter = {
    column: number;
    value: any;
    minValue?: number | null;
    maxValue?: number | null;
};
export declare type GoogleDataTableSortColumns = number | {
    column: number;
    desc: boolean;
} | number[] | {
    column: number;
    desc: boolean;
}[];
export declare type GoogleDataTable = {
    addColumn: (column: GoogleDataTableColumn) => number;
    addRow: (row?: GoogleDataTableRow) => number;
    addRows: (rows?: GoogleDataTableRow[] | number[] | any[]) => number;
    clone: () => GoogleDataTable;
    getColumnId: (columnIndex: number) => string;
    getColumnLabel: (columnIndex: number) => string;
    getColumnPattern: (columnIndex: number) => string;
    getColumnProperties: (columnIndex: number) => {};
    getColumnProperty: (columnIndex: number, name: string) => any;
    getColumnRange: (columnIndex: number) => {
        min: number | null;
        max: number | null;
    };
    getColumnRole: (columnIndex: number) => GoogleDataTableColumnRoleType;
    getColumnType: (columnIndex: number) => GoogleDataTableColumnType;
    getDistinctValues: (columnIndex: number) => any[];
    getFilteredRows: (filters: GoogleDataTableRowFilter[]) => number[];
    getFormattedValue: (rowIndex: number, columnIndex: number) => string;
    getNumberOfColumns: () => number;
    getNumberOfRows: () => number;
    getProperties: (rowIndex: number, columnIndex: number) => {};
    getProperty: (rowIndex: number, columnIndex: number, name: string) => any;
    getRowProperties: (rowIndex: number) => {};
    getRowProperty: (rowIndex: number, name: string) => any;
    getSortedRows: (sortColumns: GoogleDataTableSortColumns) => number[];
    getTableProperties: () => {};
    getTableProperty: (name: string) => any;
    getValue: (rowIndex: number, columnIndex: number) => any;
    insertColumn: (columnIndex: number, type: GoogleDataTableColumnType, label?: string, id?: string) => void;
    insertRows: (rowIndex: number, numberOrArray: GoogleDataTableRow[] | number) => void;
    removeColumn: (columnIndex: number) => void;
    removeColumns: (columnIndex: number, numberOfColumns: number) => void;
    removeRow: (rowIndex: number) => void;
    removeRows: (rowIndex: number, numberOfColumns: number) => void;
    setCell: (rowIndex: number, columnIndex: number, value?: any, formattedValue?: string, properties?: {}) => {};
    setColumnLabel: (columnIndex: number, label: string) => void;
    setColumnProperty: (columnIndex: number, name: string, value: any) => void;
    setColumnProperties: (columnIndex: number, properties: {} | null) => void;
    setFormattedValue: (rowIndex: number, columnIndex: number, formattedValue: string) => void;
    setProperty: (rowIndex: number, columnIndex: number, name: string, value: any) => void;
    setProperties: (rowIndex: number, columnIndex: number, properties: {} | null) => void;
    setRowProperty: (rowIndex: number, name: string, value: any) => void;
    setRowProperties: (rowIndex: number, properties: {} | null) => void;
    setTableProperties: (properties: {} | null) => void;
    setValue: (rowIndex: number, columnIndex: number, value: string) => void;
    sort: (sortColumns: GoogleDataTableSortColumns) => void;
    toJSON: () => string;
};
export declare type GoogleArrayToDataTable = (data: any[][], isFirstRowLabels?: boolean) => GoogleDataTable;
export declare type GoogleChartOptions = {
    width?: number;
    height?: number;
    is3D?: boolean;
    title?: string;
    hAxis?: {
        minValue?: number;
        ticks?: number[];
        title?: string;
    };
    vAxis?: {
        minValue?: number;
        ticks?: number[];
        title?: string;
    };
    bubble?: {};
    pieHole?: number;
    redFrom?: number;
    redTo?: number;
    yellowFrom?: number;
    yellowTo?: number;
    minorTicks?: number;
    legend?: string | {
        position?: string;
        maxLines?: number;
    };
    curveType?: string;
    showTooltip?: boolean;
    showInfoWindow?: boolean;
    allowHtml?: boolean;
    isStacked?: string | boolean;
    minColor?: string;
    midColor?: string;
    maxColor?: string;
    headerHeight?: number;
    fontColor?: string;
    showScale?: boolean;
    bar?: {
        groupWidth?: string;
    };
    candlestick?: {
        fallingColor?: {
            strokeWidth?: number;
            fill?: string;
        };
        risingColor?: {
            strokeWidth?: number;
            fill?: string;
        };
    };
    wordtree?: {
        format?: string;
        word?: string;
    };
};
